@@ -8,19 +8,18 @@ from inspect import BoundArguments, Parameter, Signature, signature
 from itertools import chain
 from pathlib import Path
 from typing import (
-    Any, Dict, Iterable, Iterator, List, Mapping, Optional, Sequence, Tuple,
+    Any, Callable, Dict, Iterable, Iterator, List, Mapping, Optional, Sequence, Tuple,
     Type, TypeVar, Union, cast
 )
 
 import astor
 from pydoctor import epydoc2stan, model
 
-
 def parseFile(path: Path) -> ast.Module:
     """Parse the contents of a Python source file."""
     with open(path, 'rb') as f:
         src = f.read() + b'\n'
-    return _parse(src)
+    return _parse(src, filename=str(path))
 
 if sys.version_info >= (3,8):
     _parse = partial(ast.parse, type_comments=True)
@@ -271,10 +270,13 @@ class ModuleVistor(ast.NodeVisitor):
             return
 
         modname = node.module
-        if node.level:
+        level = node.level
+        if level:
             # Relative import.
             parent: Optional[model.Documentable] = ctx.parentMod
-            for _ in range(node.level):
+            if isinstance(ctx.module, model.Package):
+                level -= 1
+            for _ in range(level):
                 if parent is None:
                     break
                 parent = parent.parent
@@ -332,13 +334,7 @@ class ModuleVistor(ast.NodeVisitor):
         """Handle a C{from <modname> import <names>} statement."""
 
         # Process the module we're importing from.
-        # If we're importing from a package, 'mod' will be the __init__ module.
         mod = self.system.getProcessedModule(modname)
-        obj = self.system.objForFullName(modname)
-
-        # Are we importing from a package?
-        is_package = isinstance(obj, model.Package)
-        assert is_package or obj is mod or mod is None
 
         # Fetch names to export.
         current = self.builder.current
@@ -375,7 +371,7 @@ class ModuleVistor(ast.NodeVisitor):
 
             # If we're importing from a package, make sure imported modules
             # are processed (getProcessedModule() ignores non-modules).
-            if is_package:
+            if isinstance(mod, model.Package):
                 self.system.getProcessedModule(f'{modname}.{orgname}')
 
             _localNameToFullName[asname] = f'{modname}.{orgname}'
@@ -404,7 +400,7 @@ class ModuleVistor(ast.NodeVisitor):
                 _localNameToFullName[asname] = fullname
 
 
-    def _handleOldSchoolDecoration(self, target: str, expr: Optional[ast.expr]) -> bool:
+    def _handleOldSchoolMethodDecoration(self, target: str, expr: Optional[ast.expr]) -> bool:
         if not isinstance(expr, ast.Call):
             return False
         func = expr.func
@@ -420,11 +416,15 @@ class ModuleVistor(ast.NodeVisitor):
         if target == arg.id and func_name in ['staticmethod', 'classmethod']:
             target_obj = self.builder.current.contents.get(target)
             if isinstance(target_obj, model.Function):
-                if target_obj.kind != 'Method':
-                    self.system.msg('ast', 'XXX')
-                else:
-                    target_obj.kind = func_name.title().replace('m', ' M')
-                    return True
+
+                # _handleOldSchoolMethodDecoration must only be called in a class scope.
+                assert target_obj.kind is model.DocumentableKind.METHOD
+
+                if func_name == 'staticmethod':
+                    target_obj.kind = model.DocumentableKind.STATIC_METHOD
+                elif func_name == 'classmethod':
+                    target_obj.kind = model.DocumentableKind.CLASS_METHOD
+                return True
         return False
 
     def _handleModuleVar(self,
@@ -433,17 +433,16 @@ class ModuleVistor(ast.NodeVisitor):
             expr: Optional[ast.expr],
             lineno: int
             ) -> None:
-        if target == '__all__':
-            # This is metadata, not a variable that needs to be documented.
-            # It is handled by findAll(), which operates on the AST and
-            # therefore doesn't need an Attribute instance.
+        if target in MODULE_VARIABLES_META_PARSERS:
+            # This is metadata, not a variable that needs to be documented,
+            # and therefore doesn't need an Attribute instance.
             return
         parent = self.builder.current
         obj = parent.resolveName(target)
         if obj is None:
-            obj = self.builder.addAttribute(target, None, parent)
+            obj = self.builder.addAttribute(name=target, kind=None, parent=parent)
         if isinstance(obj, model.Attribute):
-            obj.kind = 'Variable'
+            obj.kind = model.DocumentableKind.VARIABLE
             if annotation is None and expr is not None:
                 annotation = _infer_type(expr)
             obj.annotation = annotation
@@ -473,7 +472,7 @@ class ModuleVistor(ast.NodeVisitor):
             return
         obj: Optional[model.Attribute] = cls.contents.get(name)
         if obj is None:
-            obj = self.builder.addAttribute(name, None, cls)
+            obj = self.builder.addAttribute(name=name, kind=None, parent=cls)
         if obj.kind is None:
             instance = is_attrib(expr, cls) or (
                 cls.auto_attribs and annotation is not None and not (
@@ -481,7 +480,7 @@ class ModuleVistor(ast.NodeVisitor):
                     node2fullname(annotation.value, cls) == 'typing.ClassVar'
                     )
                 )
-            obj.kind = 'Instance Variable' if instance else 'Class Variable'
+            obj.kind = model.DocumentableKind.INSTANCE_VARIABLE if instance else model.DocumentableKind.CLASS_VARIABLE
         if expr is not None:
             if annotation is None:
                 annotation = self._annotation_from_attrib(expr, cls)
@@ -507,8 +506,8 @@ class ModuleVistor(ast.NodeVisitor):
             return
         obj = cls.contents.get(name)
         if obj is None:
-            obj = self.builder.addAttribute(name, None, cls)
-        obj.kind = 'Instance Variable'
+            obj = self.builder.addAttribute(name=name, kind=None, parent=cls)
+        obj.kind = model.DocumentableKind.INSTANCE_VARIABLE
         if annotation is None and expr is not None:
             annotation = _infer_type(expr)
         obj.annotation = annotation
@@ -586,7 +585,7 @@ class ModuleVistor(ast.NodeVisitor):
             if isinstance(scope, model.Module):
                 self._handleAssignmentInModule(target, annotation, expr, lineno)
             elif isinstance(scope, model.Class):
-                if not self._handleOldSchoolDecoration(target, expr):
+                if not self._handleOldSchoolMethodDecoration(target, expr):
                     self._handleAssignmentInClass(target, annotation, expr, lineno)
         elif isinstance(targetNode, ast.Attribute):
             value = targetNode.value
@@ -691,9 +690,9 @@ class ModuleVistor(ast.NodeVisitor):
             if is_classmethod:
                 func.report(f'{func.fullName()} is both classmethod and staticmethod')
             else:
-                func.kind = 'Static Method'
+                func.kind = model.DocumentableKind.STATIC_METHOD
         elif is_classmethod:
-            func.kind = 'Class Method'
+            func.kind = model.DocumentableKind.CLASS_METHOD
 
         # Position-only arguments were introduced in Python 3.8.
         posonlyargs: Sequence[ast.arg] = getattr(node.args, 'posonlyargs', ())
@@ -746,7 +745,7 @@ class ModuleVistor(ast.NodeVisitor):
             lineno: int
             ) -> model.Attribute:
 
-        attr = self.builder.addAttribute(node.name, 'Property', self.builder.current)
+        attr = self.builder.addAttribute(name=node.name, kind=model.DocumentableKind.PROPERTY, parent=self.builder.current)
         attr.setLineNumber(lineno)
 
         if docstring is not None:
@@ -1035,7 +1034,7 @@ class ASTBuilder:
         self._pop(self.system.Function)
 
     def addAttribute(self,
-            name: str, kind: Optional[str], parent: model.Documentable
+            name: str, kind: Optional[model.DocumentableKind], parent: model.Documentable
             ) -> model.Attribute:
         system = self.system
         parentMod = self.currentMod
@@ -1049,7 +1048,14 @@ class ASTBuilder:
         self.system._warning(self.current, message, detail)
 
     def processModuleAST(self, mod_ast: ast.Module, mod: model.Module) -> None:
-        findAll(mod_ast, mod)
+
+        for name, node in findModuleLevelAssign(mod_ast):
+            try:
+                module_var_parser = MODULE_VARIABLES_META_PARSERS[name]
+            except KeyError:
+                continue
+            else:
+                module_var_parser(node, mod)
 
         self.ModuleVistor(self, mod).visit(mod_ast)
 
@@ -1067,38 +1073,93 @@ class ASTBuilder:
 
 model.System.defaultBuilder = ASTBuilder
 
-def findAll(mod_ast: ast.Module, mod: model.Module) -> None:
-    """Find and attempt to parse into a list of names the __all__ of a module's AST."""
+def findModuleLevelAssign(mod_ast: ast.Module) -> Iterator[Tuple[str, ast.Assign]]:
+    """
+    Find module level Assign. 
+    Yields tuples containing the assigment name and the Assign node.
+    """
     for node in mod_ast.body:
         if isinstance(node, ast.Assign) and \
-               len(node.targets) == 1 and \
-               isinstance(node.targets[0], ast.Name) and \
-               node.targets[0].id == '__all__':
-            if not isinstance(node.value, (ast.List, ast.Tuple)):
-                mod.report(
-                    'Cannot parse value assigned to "__all__"',
-                    section='all', lineno_offset=node.lineno)
-                continue
+            len(node.targets) == 1 and \
+            isinstance(node.targets[0], ast.Name):
+                yield (node.targets[0].id, node)
 
-            names = []
-            for idx, item in enumerate(node.value.elts):
-                try:
-                    name: object = ast.literal_eval(item)
-                except ValueError:
-                    mod.report(
-                        f'Cannot parse element {idx} of "__all__"',
-                        section='all', lineno_offset=node.lineno)
-                else:
-                    if isinstance(name, str):
-                        names.append(name)
-                    else:
-                        mod.report(
-                            f'Element {idx} of "__all__" has '
-                            f'type "{type(name).__name__}", expected "str"',
-                            section='all', lineno_offset=node.lineno)
+def parseAll(node: ast.Assign, mod: model.Module) -> None:
+    """Find and attempt to parse into a list of names the 
+    C{__all__} variable of a module's AST and set L{Module.all} accordingly."""
 
-            if mod.all is not None:
+    if not isinstance(node.value, (ast.List, ast.Tuple)):
+        mod.report(
+            'Cannot parse value assigned to "__all__"',
+            section='all', lineno_offset=node.lineno)
+        return
+
+    names = []
+    for idx, item in enumerate(node.value.elts):
+        try:
+            name: object = ast.literal_eval(item)
+        except ValueError:
+            mod.report(
+                f'Cannot parse element {idx} of "__all__"',
+                section='all', lineno_offset=node.lineno)
+        else:
+            if isinstance(name, str):
+                names.append(name)
+            else:
                 mod.report(
-                    'Assignment to "__all__" overrides previous assignment',
+                    f'Element {idx} of "__all__" has '
+                    f'type "{type(name).__name__}", expected "str"',
                     section='all', lineno_offset=node.lineno)
-            mod.all = names
+
+    if mod.all is not None:
+        mod.report(
+            'Assignment to "__all__" overrides previous assignment',
+            section='all', lineno_offset=node.lineno)
+    mod.all = names
+
+def parseDocformat(node: ast.Assign, mod: model.Module) -> None:
+    """
+    Find C{__docformat__} variable of this 
+    module's AST and set L{Module.docformat} accordingly.
+        
+    This is all valid::
+
+        __docformat__ = "reStructuredText en"
+        __docformat__ = "epytext"
+        __docformat__ = "restructuredtext"
+    """
+
+    try:
+        value = ast.literal_eval(node.value)
+    except ValueError:
+        mod.report(
+            'Cannot parse value assigned to "__docformat__": not a string',
+            section='docformat', lineno_offset=node.lineno)
+        return
+    
+    if not isinstance(value, str):
+        mod.report(
+            'Cannot parse value assigned to "__docformat__": not a string',
+            section='docformat', lineno_offset=node.lineno)
+        return
+        
+    if not value.strip():
+        mod.report(
+            'Cannot parse value assigned to "__docformat__": empty value',
+            section='docformat', lineno_offset=node.lineno)
+        return
+    
+    # Language is ignored and parser name is lowercased.
+    value = value.split(" ", 1)[0].lower()
+
+    if mod._docformat is not None:
+        mod.report(
+            'Assignment to "__docformat__" overrides previous assignment',
+            section='docformat', lineno_offset=node.lineno)
+
+    mod.docformat = value
+
+MODULE_VARIABLES_META_PARSERS: Mapping[str, Callable[[ast.Assign, model.Module], None]] = {
+    '__all__': parseAll,
+    '__docformat__': parseDocformat
+}
