@@ -2,12 +2,16 @@
 
 from optparse import SUPPRESS_HELP, Option, OptionParser, OptionValueError, Values
 from pathlib import Path
-from typing import TYPE_CHECKING, List, Sequence, Tuple, Type, TypeVar, cast
+from typing import Iterator, TYPE_CHECKING, List, Sequence, Tuple, Type, TypeVar, cast
 import datetime
 import os
 import sys
+import warnings
+from inspect import getmodulename
 
+from pydoctor.themes import get_themes
 from pydoctor import model, zopeinterface, __version__
+from pydoctor.templatewriter import IWriter, TemplateError, TemplateLookup
 from pydoctor.sphinx import (MAX_AGE_HELP, USER_INTERSPHINX_CACHE,
                              SphinxInventoryWriter, prepareCache)
 
@@ -16,6 +20,11 @@ if TYPE_CHECKING:
 else:
     NoReturn = None
 
+# Newer APIs from importlib_resources should arrive to stdlib importlib.resources in Python 3.9.
+if sys.version_info < (3, 9):
+    import importlib_resources
+else:
+    import importlib.resources as importlib_resources
 
 BUILDTIME_FORMAT = '%Y-%m-%d %H:%M:%S'
 
@@ -75,6 +84,17 @@ def parse_path(option: Option, opt: str, value: str) -> Path:
     except Exception as ex:
         raise OptionValueError(f"{opt}: invalid path: {ex}")
 
+def get_supported_docformats() -> Iterator[str]:
+    """
+    Get the list of currently supported docformat.
+    """
+    for fileName in importlib_resources.contents('pydoctor.epydoc.markup'):
+        moduleName = getmodulename(fileName)
+        if moduleName is None or moduleName == '__init__':
+            continue
+        else:
+            yield moduleName
+
 class CustomOption(Option):
     TYPES = Option.TYPES + ("path",)
     TYPE_CHECKER = dict(Option.TYPE_CHECKER, path=parse_path)
@@ -131,14 +151,25 @@ def getparser() -> OptionParser:
         '--prepend-package', action='store', dest='prependedpackage',
         help=("Pretend that all packages are within this one.  "
               "Can be used to document part of a package."))
+    _docformat_choices = get_supported_docformats()
     parser.add_option(
         '--docformat', dest='docformat', action='store', default='epytext',
-        help=("Which epydoc-supported format docstrings are assumed "
-              "to be in."))
+        type="choice", choices=list(_docformat_choices),
+        help=("Format used for parsing docstrings. "
+             f"Supported values: {', '.join(_docformat_choices)}"))
+    parser.add_option(
+        '--template-dir', action='append',
+        dest='templatedir', default=[],
+        help=("Directory containing custom HTML templates. Can repeat."),
+    )
+    parser.add_option('--theme', dest='theme', default='classic', 
+        choices=list(get_themes()) ,
+        help=("The theme to use when building your API documentation. "),
+    )
     parser.add_option(
         '--html-subject', dest='htmlsubjects', action='append',
-        help=("The fullName of object to generate API docs for"
-              " (default: everything)."))
+        help=("The fullName of objects to generate API docs for"
+              " (generates everything by default)."))
     parser.add_option(
         '--html-summary-pages', dest='htmlsummarypages',
         action='store_true', default=False,
@@ -148,7 +179,7 @@ def getparser() -> OptionParser:
         help=("Directory to save HTML files to (default 'apidocs')"))
     parser.add_option(
         '--html-writer', dest='htmlwriter',
-        help=("Dotted name of html writer class to use (default "
+        help=("Dotted name of HTML writer class to use (default "
               "'pydoctor.templatewriter.TemplateWriter')."))
     parser.add_option(
         '--html-viewsource-base', dest='htmlsourcebase',
@@ -369,10 +400,12 @@ def main(args: Sequence[str] = sys.argv[1:]) -> int:
                         error(f"Source path lies outside base directory: {ex}")
                 if path.is_dir():
                     system.msg('addPackage', f"adding directory {path}")
-                    system.addPackage(str(path), prependedpackage)
+                    if not (path / '__init__.py').is_file():
+                        error(f"Source directory lacks __init__.py: {path}")
+                    system.addPackage(path, prependedpackage)
                 elif path.is_file():
                     system.msg('addModuleFromPath', f"adding module {path}")
-                    system.addModuleFromPath(prependedpackage, str(path))
+                    system.addModuleFromPath(path, prependedpackage)
                 elif path.exists():
                     error(f"Source path is neither file nor directory: {path}")
                 else:
@@ -384,7 +417,7 @@ def main(args: Sequence[str] = sys.argv[1:]) -> int:
         # step 3: move the system to the desired state
 
         if system.options.projectname is None:
-            name = '/'.join(ro.name for ro in system.rootobjects)
+            name = '/'.join(system.root_names)
             system.msg('warning', f"Guessing '{name}' for project name.", thresh=0)
             system.projectname = name
         else:
@@ -392,6 +425,7 @@ def main(args: Sequence[str] = sys.argv[1:]) -> int:
 
         system.process()
 
+
         # step 4: make html, if desired
 
         if options.makehtml:
@@ -399,8 +433,7 @@ def main(args: Sequence[str] = sys.argv[1:]) -> int:
             from pydoctor import templatewriter
             if options.htmlwriter:
                 writerclass = findClassFromDottedName(
-                    options.htmlwriter, '--html-writer',
-                    templatewriter.TemplateWriter)
+                    options.htmlwriter, '--html-writer', IWriter)
             else:
                 writerclass = templatewriter.TemplateWriter
 
@@ -408,19 +441,47 @@ def main(args: Sequence[str] = sys.argv[1:]) -> int:
                 options.htmloutput, writerclass.__module__,
                 writerclass.__name__))
 
-            writer = writerclass(options.htmloutput)
+            writer: IWriter
+            
+            # Always init the writer with the 'base' set of templates at least.
+            template_lookup = TemplateLookup(
+                                importlib_resources.files('pydoctor.themes') / 'base')
+            
+            # Handle theme selection, 'classic' by default.
+            if system.options.theme != 'base':
+                template_lookup.add_templatedir(
+                    importlib_resources.files('pydoctor.themes') / system.options.theme)
+
+
+            # Handle custom HTML templates
+            if system.options.templatedir:
+                try:
+                    for t in system.options.templatedir:
+                        template_lookup.add_templatedir(Path(t))
+                except TemplateError  as e:
+                    error(str(e))
+
+            build_directory = Path(options.htmloutput)
+
+            try:
+                # mypy error: Cannot instantiate abstract class 'IWriter'
+                writer = writerclass(build_directory, # type: ignore[abstract]
+                    template_lookup=template_lookup)
+            except TypeError:
+                # Custom class does not accept 'template_lookup' argument.
+                writer = writerclass(build_directory) # type: ignore[abstract]
+                warnings.warn(f"Writer '{writerclass.__name__}' does not support "
+                    "HTML template customization with --template-dir.")
+
             writer.prepOutputDirectory()
 
+            subjects: Sequence[model.Documentable] = ()
             if options.htmlsubjects:
-                subjects = []
-                for fn in options.htmlsubjects:
-                    subjects.append(system.allobjects[fn])
-            elif options.htmlsummarypages:
-                writer.writeModuleIndex(system)
-                subjects = []
+                subjects = [system.allobjects[fn] for fn in options.htmlsubjects]
             else:
-                writer.writeModuleIndex(system)
-                subjects = system.rootobjects
+                writer.writeSummaryPages(system)
+                if not options.htmlsummarypages:
+                    subjects = system.rootobjects
             writer.writeIndividualFiles(subjects)
             if system.docstring_syntax_errors:
                 def p(msg: str) -> None:
