@@ -33,11 +33,16 @@ each error.
 """
 __docformat__ = 'epytext en'
 
-from typing import List, Optional, Sequence, Union
-import re
+from typing import TYPE_CHECKING, Optional, Sequence
+import abc
 
-from twisted.python.failure import Failure
-from twisted.web.template import Tag, XMLString, flattenString
+from docutils import nodes
+from twisted.web.template import Tag
+
+if TYPE_CHECKING:
+    from twisted.web.template import Flattenable
+
+from pydoctor import node2stan
 
 ##################################################
 ## Contents
@@ -52,14 +57,14 @@ from twisted.web.template import Tag, XMLString, flattenString
 ##################################################
 ## ParsedDocstring
 ##################################################
-class ParsedDocstring:
+class ParsedDocstring(abc.ABC):
     """
     A standard intermediate representation for parsed docstrings that
     can be used to generate output.  Parsed docstrings are produced by
     markup parsers such as L{pydoctor.epydoc.markup.epytext.parse_docstring()}
     or L{pydoctor.epydoc.markup.restructuredtext.parse_docstring()}.
 
-    Subclasses must implement L{has_body()} and L{to_stan()}.
+    Subclasses must implement L{has_body()} and L{to_node()}.
     """
 
     def __init__(self, fields: Sequence['Field']):
@@ -69,7 +74,9 @@ class ParsedDocstring:
         The field's bodies are encoded as C{ParsedDocstring}s.
         """
 
-    @property
+        self._stan: Optional[Tag] = None
+
+    @abc.abstractproperty
     def has_body(self) -> bool:
         """Does this docstring have a non-empty body?
 
@@ -82,53 +89,27 @@ class ParsedDocstring:
         """
         Translate this docstring to a Stan tree.
 
-        Implementations are encouraged to generate Stan output directly
-        if possible, but if that is not feasible, L{html2stan()} can be
-        used instead.
+        @note: The default implementation relies on functionalities 
+            provided by L{node2stan.node2stan} and L{ParsedDocstring.to_node()}.
 
         @param docstring_linker: An HTML translator for crossreference
             links into and out of the docstring.
-        @return: The docstring presented as a tree.
+        @return: The docstring presented as a stan tree.
+        """
+        if self._stan is not None:
+            return self._stan
+        self._stan = Tag('', children=node2stan.node2stan(self.to_node(), docstring_linker).children)
+        return self._stan
+    
+    @abc.abstractmethod
+    def to_node(self) -> nodes.document:
+        """
+        Translate this docstring to a L{docutils.nodes.document}.
+
+        @return: The docstring presented as a L{docutils.nodes.document}.
         """
         raise NotImplementedError()
 
-_RE_CONTROL = re.compile((
-    '[' + ''.join(
-    ch for ch in map(chr, range(0, 32)) if ch not in '\r\n\t\f'
-    ) + ']'
-    ).encode())
-
-def html2stan(html: Union[bytes, str]) -> Tag:
-    """
-    Convert an HTML string to a Stan tree.
-
-    @param html: An HTML fragment; multiple roots are allowed.
-    @return: The fragment as a tree with a transparent root node.
-    """
-    if isinstance(html, str):
-        html = html.encode('utf8')
-
-    html = _RE_CONTROL.sub(lambda m:b'\\x%02x' % ord(m.group()), html)
-    stan: Tag = XMLString(b'<div>%s</div>' % html).load()[0]
-    assert stan.tagName == 'div'
-    stan.tagName = ''
-    return stan
-
-def flatten(stan: Tag) -> str:
-    """
-    Convert a document fragment from a Stan tree to HTML.
-
-    @param stan: Document fragment to flatten.
-    @return: An HTML string representation of the C{stan} tree.
-    """
-    ret: List[bytes] = []
-    err: List[Failure] = []
-    flattenString(None, stan).addCallback(ret.append).addErrback(err.append)
-    if err:
-        raise err[0].value
-    else:
-        return ret[0].decode()
-
 ##################################################
 ## Fields
 ##################################################
@@ -188,7 +169,7 @@ class DocstringLinker:
     target URL for crossreference links.
     """
 
-    def link_to(self, target: str, label: str) -> Tag:
+    def link_to(self, target: str, label: "Flattenable") -> Tag:
         """
         Format a link to a Python identifier.
         This will resolve the identifier like Python itself would.
@@ -200,7 +181,7 @@ class DocstringLinker:
         """
         raise NotImplementedError()
 
-    def link_xref(self, target: str, label: str, lineno: int) -> Tag:
+    def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
         """
         Format a cross-reference link to a Python identifier.
         This will resolve the identifier to any reasonable target,
