@@ -1,15 +1,18 @@
 """PyDoctor's test suite."""
 
 from logging import LogRecord
-from typing import TYPE_CHECKING, Optional, Sequence
+from typing import Iterable, TYPE_CHECKING, Optional, Sequence
 import sys
 import pytest
 
 from twisted.web.template import Tag, tags
 
 from pydoctor import epydoc2stan, model
+from pydoctor.templatewriter import IWriter
 from pydoctor.epydoc.markup import DocstringLinker
 
+if TYPE_CHECKING:
+    from twisted.web.template import Flattenable
 
 posonlyargs = pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python 3.8")
 typecomment = pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python 3.8")
@@ -39,7 +42,7 @@ else:
     FixtureRequest = MonkeyPatch = TempPathFactory = object
 
 
-class InMemoryWriter:
+class InMemoryWriter(IWriter):
     """
     Minimal template writer that doesn't touches the filesystem but will
     trigger the rendering of epydoc for the targeted code.
@@ -53,14 +56,14 @@ class InMemoryWriter:
         Does nothing.
         """
 
-    def writeIndividualFiles(self, obs: Sequence[model.Documentable]) -> None:
+    def writeIndividualFiles(self, obs: Iterable[model.Documentable]) -> None:
         """
         Trigger in memory rendering for all objects.
         """
         for ob in obs:
             self._writeDocsFor(ob)
 
-    def writeModuleIndex(self, system: model.System) -> None:
+    def writeSummaryPages(self, system: model.System) -> None:
         """
         Rig the system to not created the inter sphinx inventory.
         """
@@ -82,11 +85,11 @@ class InMemoryWriter:
 class NotFoundLinker(DocstringLinker):
     """A DocstringLinker implementation that cannot find any links."""
 
-    def link_to(self, target: str, label: str) -> Tag:
-        return tags.transparent(label)  # type: ignore[no-any-return]
+    def link_to(self, target: str, label: "Flattenable") -> Tag:
+        return tags.transparent(label)
 
-    def link_xref(self, target: str, label: str, lineno: int) -> Tag:
-        return tags.code(label)  # type: ignore[no-any-return]
+    def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
+        return tags.code(label)
 
     def resolve_identifier(self, identifier: str) -> Optional[str]:
         return None
